/* QLogic (R)NIC Driver/Library
 * Copyright (c) 2010-2017  Cavium, Inc.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef _QED_ROCE_H
#define _QED_ROCE_H
#include <linux/types.h>
#include <linux/slab.h>

#define QED_ROCE_QP_TO_ICID(qp_idx)             ((qp_idx) * 2)
#define QED_ROCE_ICID_TO_QP(icid)               ((icid) / 2)

#ifdef CONFIG_DCQCN

/* According to 802-1au */
#define DCQCN_RPG_AI_RATE       5
#define DCQCN_RPG_HAI_RATE      50
#define DCQCN_RPG_GD            256

/* According to sigcomm */
#define DCQCN_CNP_SEND_TIMEOUT  50
#define DCQCN_BYTECOUNT_RATE    (10 * 1024 * 1024)
#define DCQCN_ALPHA_INTERVAL    55
#define DCQCN_TIMEOUT           55

#define DCQCN_RL_BC_STAGE_TH    5
#define DCQCN_RL_TIMER_STAGE_TH 5

/* 100G speed */
#define DCQCN_RL_MAX_RATE       100000

#endif

/* functions for enabling/disabling edpm in rdma PFs according to existence of
 * qps during DCBx update or bar size
 */
void qed_roce_dpm_dcbx(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt);

void qed_rdma_dpm_bar(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt);

void qed_dcqcn_info_init(struct qed_hwfn *p_hwfn);

int
qed_roce_dcqcn_cfg(struct qed_hwfn *p_hwfn,
		   struct roce_init_func_ramrod_data *p_ramrod,
		   struct qed_ptt *p_ptt);

int
qed_roce_dcqcn_update(struct qed_hwfn *p_hwfn,
		      struct qed_roce_dcqcn_params *p_dcqcn_params);

int qed_roce_setup(struct qed_hwfn *p_hwfn);

int qed_roce_stop_rl(struct qed_hwfn *p_hwfn);

int qed_roce_stop(struct qed_hwfn *p_hwfn);

int qed_roce_start_rl(struct qed_hwfn *p_hwfn,
		      struct qed_roce_dcqcn_params *dcqcn_params);

int
qed_roce_query_qp(struct qed_hwfn *p_hwfn,
		  struct qed_rdma_qp *qp,
		  struct qed_rdma_query_qp_out_params *out_params);

int
qed_roce_destroy_qp(struct qed_hwfn *p_hwfn,
		    struct qed_rdma_qp *qp,
		    struct qed_rdma_destroy_qp_out_params *out_params);

int qed_roce_alloc_qp_idx(struct qed_hwfn *p_hwfn, u16 * qp_idx16);

#define IS_QED_DCQCN(p_hwfn) \
	(!!(p_hwfn->pf_params.rdma_pf_params.enable_dcqcn))

struct qed_roce_info {
	struct roce_events_stats event_stats;
	struct roce_dcqcn_received_stats dcqcn_rx_stats;
	struct roce_dcqcn_sent_stats dcqcn_tx_stats;
	struct qed_roce_dcqcn_params dcqcn_params;
	struct roce_cqe_stats cqe_stats;
	struct roce_error_stats error_stats;
};

int
qed_roce_modify_qp(struct qed_hwfn *p_hwfn,
		   struct qed_rdma_qp *qp,
		   enum qed_roce_qp_state prev_state,
		   struct qed_rdma_modify_qp_in_params *params);

#endif
